/*
 * Project name:
     Arduino MEGA Click Shield Demonstration
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150821:
       - initial release (DR);   
 * Description:
     This project features a simple demonstration of the Click Shield. The example uses HDC1000 Click board
     to measure temperature and humidity, 8x8 Click board to display the measuring results and RELAY Click Board
     that turns relays on/off according to the temperature/humidity values set in the code.
 * Test configuration:
     MCU:             ATmega2560
                      http://www.atmel.com/images/atmel-2549-8-bit-avr-microcontroller-atmega640-1280-1281-2560-2561_datasheet.pdf
     Dev. Board:      Arduino MEGA 2560
                      https://www.arduino.cc/en/Main/ArduinoBoardMega2560
     Oscillator:      16.0000 MHz Crystal
     Ext. Modules:    HDC1000 Click board
     			            http://www.mikroe.com/click/hdc1000/
     									8x8 Click board
                      http://www.mikroe.com/click/8x8-g/
                      RELAY Click board 
                      http://www.mikroe.com/click/relay/
                      Arduino MEGA Click Shield
                      http://www.mikroe.com/click/arduino-mega-shield
     SW:              Arduino IDE
                      http://arduino.cc/en/main/software
 * NOTES:
     - Place 8x8 Click into mikroBUS socket 1 Arduino MEGA Click Shield.
     - Place HDC1000 Click into mikroBUS socket 2 Arduino MEGA Click Shield.
     - Place RELAY Click into mikroBUS socket 1 Arduino MEGA Click Shield.     
     - Set the on-board to PROG position when uploading project.
 */

#include "font_8x8.h"
#include <SPI.h>
#include <Wire.h>

#define DELAY 50
#define Tconstant 0.0025177

// Click board connections
#define Chip_Select1 53 // socket 1
#define relay1 3
#define relay2 45
// Click board connections

char i;
char transImage1[8]; 
char transImage2[8]; 
char transImage3[8]; 
char transImage4[8];

char tmp_data[3];
char charVal[10];

float temperature;
float humidity;

void setup() 
{
    pinMode(Chip_Select1, OUTPUT);  // define socket 1 chip select pin as output 
    pinMode(relay1, OUTPUT);
    pinMode(relay2, OUTPUT);
    // Initialize SPI1 module
    SPI.begin();
    max7219_init();  
    Wire.begin(); 
    delay(5);
    sensor_setup();
}

void loop() 
{ 
    Read_Temperature();
    Read_Humidity();
    
    Relay_management();
    Writing_output();
}

// set up MAX7219 for defined mikrobus socket

void max7219_init() 
{
    digitalWrite(Chip_Select1, LOW);  // SELECT MAX
    SPI.transfer(0x09);               // No decoding
    SPI.transfer(0x00);
    digitalWrite(Chip_Select1, HIGH); // DESELECT MAX

    digitalWrite(Chip_Select1, LOW);  // SELECT MAX
    SPI.transfer(0x0A);
    SPI.transfer(0x0a);               // Segment luminosity intensity
    digitalWrite(Chip_Select1, HIGH); // DESELECT MAX

    digitalWrite(Chip_Select1, LOW);  // SELECT MAX
    SPI.transfer(0x0B);
    SPI.transfer(0x07);               // Display refresh
    digitalWrite(Chip_Select1, HIGH); // DESELECT MAX

    digitalWrite(Chip_Select1, LOW);  // SELECT MAX
    SPI.transfer(0x0C);
    SPI.transfer(0x01);               // Turn on the display
    digitalWrite(Chip_Select1, HIGH); // DESELECT MAX

    digitalWrite(Chip_Select1, LOW);  // SELECT MAX
    SPI.transfer(0x00);
    SPI.transfer(0xFF);               // No test
    digitalWrite(Chip_Select1, HIGH); // DESELECT MAX
}

void display_column(char col, char col_num)
{
    digitalWrite(Chip_Select1, LOW);  // select max7219
    SPI.transfer(col_num+1);
    SPI.transfer(col);
    digitalWrite(Chip_Select1, HIGH); // deselect max7219 
}

void display_image_const(const char *image) 
{
    int i;

    for(i = 7; i>=0; i--) 
    {             
        display_column(image[i], i);
    }
}

void display_image(char *image) 
{
    int i;

    for(i = 7; i>=0; i--) 
    {                    
        display_column(image[i], i);
    }
}

void slide_text_double(const char *slide) 
{
    int i, j;
    int cntr;
    int tmp;
    char image1, image2, image3;

    display_image_const(font+(*slide << 3));
    delay(DELAY);

    cntr = 0;
    while (slide[cntr+1]) 
    {
        image1 = *(slide+cntr);
        image2 = *(slide+cntr+1);
        image3 = *(slide+cntr+2);
    
        for (i=0; i<8; i++) 
        {
            tmp = (image1 << 3) + i;
            transImage1[i] = *(font+tmp);
            tmp = (image2 << 3) + i;
            transImage2[i] = *(font+tmp);
            transImage4[i] = *(font+tmp);
            tmp = (image3 << 3) + i;
            transImage3[i] = *(font+tmp);
        }
    
        for (j=7; j>=0; j--)
        {
            for (i=7; i>=1; i--)
            {
                transImage1[i] = transImage1[i-1];
                transImage2[i] = transImage2[i-1];
            }
            transImage1[0] = transImage4[j];
            transImage2[0] = transImage3[j];
            display_image(transImage1);
            delay(DELAY);
        }
        cntr++;
    }
}

static void sensor_setup() 
{
    tmp_data[0] = 0x02;
    tmp_data[1] = 0x08;
    tmp_data[2] = 0x00;
    Wire.beginTransmission(0x40);
    Wire.write(tmp_data,3);
    Wire.endTransmission();
}
static void Read_Humidity()
{
    unsigned int humi_value = 0;
    tmp_data[0] = 0x01;
    Wire.beginTransmission(0x40);
    Wire.write(tmp_data,1);
    Wire.endTransmission();
    delay(20);
    Wire.requestFrom(0x40,2);
    if(Wire.available()>=2)
    {
        humi_value = Wire.read()<<8;
        humi_value += Wire.read();
    }
    humidity = humi_value/65536.0*100.0;
}
static void Read_Temperature() 
{
    unsigned int temp_value = 0;
    tmp_data[0] = 0x00;
    Wire.beginTransmission(0x40);
    Wire.write(tmp_data,1);
    Wire.endTransmission();
    delay(20);
    Wire.requestFrom(0x40,2);
    if(Wire.available()>=2)
    {
        temp_value = Wire.read()<<8;
        temp_value += Wire.read();
    }
    temperature = ((float)temp_value * Tconstant)-40;
}
static void Relay_management()
{
    if(temperature>30)
        digitalWrite(relay1, HIGH);
    else
        digitalWrite(relay1, LOW);
    if(humidity>80)
        digitalWrite(relay2, HIGH);
    else
        digitalWrite(relay2, LOW);
}
static void Writing_output()
{
    dtostrf(temperature, 3, 1, charVal+1);
    charVal[5] = 127;
    charVal[6] = 'C';
    charVal[7] = ' ';    
    charVal[8] = 0; 
    slide_text_double(charVal);

    dtostrf(humidity, 3, 1, charVal+1);
    charVal[0] = ' ';
    charVal[5] = 37;
    charVal[6] = ' ';
    charVal[7] = 0;
    slide_text_double(charVal);
}
